/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui;

import map.*;
import util.*;

import java.util.*;
import java.awt.image.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;



/* ====================================================================== */
/** This is the JDialog is designed to be a simple display Window for
 *  display a User's (not access point) information.  Some of the information
 *  can be edited at this point.  Also a photo, of the user's real world
 *  location is displayed if exists.
 *
 */
public class RoomDialog extends JDialog implements DocumentListener, ChangeListener{

	/** Layout used in this panel. */
    private BorderLayout bLayMain = new BorderLayout();

    /** Layout used in this panel. */
    private GridBagLayout gbagLayInfo = new GridBagLayout();

	/** Main panel to hold all components */
    private JPanel pnlMain = new JPanel();

    /** The parent Frame that owns this JDialog. */
    private Frame parentFrame;


	/** An error notification if user types letters in the duration column. */
	private JDialog errorDialog;

	/** Lots of fields for User's stuff. */
	private JTextField p2Field = new JTextField(14);
	private JTextField p1Field = new JTextField(14);

	private JSlider oxySlider = new JSlider(0,100);
	private JSlider visSlider = new JSlider(0,100);

	/** A button to close this dialog. */
	private JButton btnClose = new JButton("Close");

	/** The image of the user's location if exists. */
	private ImageIcon icoLogo = new ImageIcon("files/textures/nopic.gif");

	/** A label for the user's location photo. */
	private JLabel lblPicture = new JLabel(icoLogo);

	private Room room;

    /* ===================================================================== */
    /** Constructor that creates this JDialog. The parent and controller must
     *  not be null for this Dialog to function properly.
     *
     *  @param  parent   The owning frame of this dialog.
     *  @param  controller   The simulation controller.
     *
     */
    public RoomDialog(Room room, Frame parent ) {

		super(parent);
        setTitle("Room Properties");

		this.room = room;
        this.parentFrame = parent;

		// Setup the gui
        try {
          init();
        }
        catch(Exception e) {
          e.printStackTrace();
        }


		// Set the size and location
        this.pack();
        this.setSize(620, 360);

		//Center this dialog on the screen.
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int x = ( screen.width / 2) - (this.getSize().width /2 );
		int y = ( screen.height / 2) - (this.getSize().height /2 );
        this.setLocation(x,y);

		// Show us.
        this.setVisible(true);

    } //Construtor




    // =======================================================================
    /** Initializes the GUI with components.
     *
     */
    private void init() throws Exception {

		// Setup layouts and borders
        bLayMain.setHgap(5);
        bLayMain.setVgap(5);
        pnlMain.setBorder(TTBorder.getBorder("Properties"));
        this.getContentPane().setLayout(bLayMain);
        pnlMain.setLayout(new BorderLayout());

		JPanel btnPanel = new JPanel();
        btnPanel.setLayout(gbagLayInfo);

        btnPanel.add(btnClose, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(5, 5, 5, 5), 0, 0));

		// Setup the action listeners
		btnClose.setMnemonic(KeyEvent.VK_C);
        btnClose.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  btdClose_actionPerformed();
              }
        });



		JLabel lblP1 = new JLabel("Sound level:");
		JLabel lblP2 = new JLabel("Temperature:");


		JLabel lbloxy = new JLabel("Oxygen level:");
		JLabel lblvis = new JLabel("Visibility:");



		oxySlider.setMajorTickSpacing(5);
		oxySlider.setPaintTicks(true);
		oxySlider.setSnapToTicks(true);
		oxySlider.setPaintLabels(true);
		oxySlider.addChangeListener(this);

		Hashtable labelTable = new Hashtable();
		labelTable.put(new Integer(0), new JLabel("0%") );
		labelTable.put(new Integer(100), new JLabel("100%") );
		oxySlider.setLabelTable( labelTable );

		visSlider.setMajorTickSpacing(5);
		visSlider.setPaintTicks(true);
		visSlider.setSnapToTicks(true);
		visSlider.setPaintLabels(true);
		visSlider.addChangeListener(this);

		labelTable = new Hashtable();
		labelTable.put(new Integer(0), new JLabel("0%") );
		labelTable.put(new Integer(100), new JLabel("100%") );
		visSlider.setLabelTable( labelTable );



		// Add stuff to panels
		JPanel stats = new JPanel();
		stats.setLayout(gbagLayInfo);

        pnlMain.add(stats, BorderLayout.EAST);
        pnlMain.add(lblPicture, BorderLayout.WEST);
        this.getContentPane().add(pnlMain, BorderLayout.CENTER);
        this.getContentPane().add(btnPanel, BorderLayout.SOUTH);


        stats.add(lblP1,    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblP2,     new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));


        stats.add(lblvis,    new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lbloxy,   new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));




        stats.add(p1Field,    new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(p2Field,     new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(visSlider,     new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(oxySlider,     new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));





		// Make our size fixed.
        this.setResizable(false);

		refresh();

    } //method: init



    /* ===================================================================== */
    /** This method is from being a ControllerListener, everytime the Controller
     *  notifies us, we refresh the user information in the all fields, and check
     *  for a new photo of the current location and heading.
     *
     */
	public void refresh(){


		p1Field.getDocument().removeDocumentListener(this);
		p2Field.getDocument().removeDocumentListener(this);
		oxySlider.removeChangeListener(this);
		visSlider.removeChangeListener(this);


		p1Field.setText( ""+room.getSound() );
		p2Field.setText( ""+room.getTemperature() );

		visSlider.setValue((int)(room.getVisibility()*100));
		oxySlider.setValue((int)(room.getOxygen()*100));

		p1Field.getDocument().addDocumentListener(this);
		p2Field.getDocument().addDocumentListener(this);
		oxySlider.addChangeListener(this);
		visSlider.addChangeListener(this);




		try{

			icoLogo = new ImageIcon("files/textures/"+room.getName());
			if( icoLogo.getIconHeight() < 10 ){

				icoLogo = new ImageIcon("files/textures/nopic.gif");
			}

			lblPicture.setIcon(icoLogo);


		}catch(Exception exc){

			icoLogo = new ImageIcon("files/textures/nopic.gif");
			lblPicture.setIcon(icoLogo);
		}

	} //method: refresh



	public void stateChanged(ChangeEvent e){

		room.setVisibility( visSlider.getValue()/100.0 );
		room.setOxygen( oxySlider.getValue()/100.0 );

	}



   	/* ====================================================================== */
	/** This method is called whenever user edits a value in JTextField.
	 *
	 *  @param  e  The document event from one of our textfields.
	 *
	 */
    public void insertUpdate(DocumentEvent e) {

		//Try to grab text from textfields and save it to the current cd.
   	 	try{

			//set these values.  year must be an int or else it will throw
			// exception.

			room.setSound( Double.parseDouble(p1Field.getText()) );
			room.setTemperature( Double.parseDouble(p2Field.getText()) );


			//controller.removeControllerListener(this);
			//controller.fireControllerChange();
			//controller.addControllerListener(this);

		//This exception is thrown if the uses types a non-numeric character
		// into the duration column.
		}catch(NumberFormatException nfe){

			//This is outdated. Just use java's predefined dialog for this.

			//Setup a current/new JDialog.
			errorDialog = new JDialog(this, "Error");

			//Create a button and the labels.
			JLabel lbl = new JLabel("Please only enter numbers in the fields.");
			JLabel lblPic = new JLabel( new ImageIcon("files/textures/info.jpg") );
			JButton btn = new JButton( "OK");

			//Create panels for hold the Components
			JPanel btnpanel = new JPanel();
			JPanel lblpanel = new JPanel();
			lblpanel.setBorder( BorderFactory.createEmptyBorder(15, 15, 5, 15) );

			//Add components to the panels, then add panels to the JDialog.
			lblpanel.add(lblPic, BorderLayout.WEST);
			lblpanel.add(lbl, BorderLayout.EAST);
			btnpanel.add(btn);

			errorDialog.getContentPane().add(lblpanel, BorderLayout.CENTER);
			errorDialog.getContentPane().add(btnpanel, BorderLayout.SOUTH);

			//Make a button listener to close the dialog.
			btn.setMnemonic(KeyEvent.VK_O);
			btn.addActionListener(new ActionListener() {
            	public void actionPerformed(ActionEvent e) {
            	    errorDialog.dispose();
            	}
        	});

			//Pack it all together
        	errorDialog.pack();

			//Center this dialog on the screen.
			Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
			int x = ( screen.width / 2) - (errorDialog.getSize().width /2 );
			int y = ( screen.height / 2) - (errorDialog.getSize().height /2 );
			errorDialog.setLocation( x, y);

			//Now show this dialog.
        	errorDialog.setVisible(true);


			//Now make sure dialog is in front and in focus.
        	errorDialog.toFront();


		} //try

    } //method: insertUpdate


   	/* ====================================================================== */
	/** Don't care about this method.  Only need it to satisfy DocumentListener
	 *  interface.
	 *
	 *  @param  e  The event we don't care about.
	 *
	 */
    public void removeUpdate(DocumentEvent e){}
    /* ====================================================================== */
 	/** Don't care about this method.  Only need it to satisfy DocumentListener
 	 *  interface.
 	 *
 	 *  @param  e  The event we don't care about.
 	 *
	 */
 	public void changedUpdate(DocumentEvent e){}
    /* ====================================================================== */
 	/** Don't care about this method.  Only need it to satisfy DocumentListener
 	 *  interface.
 	 *
 	 *  @param  e  The event we don't care about.
 	 *
	 */
 	private void calculateValue(DocumentEvent e){}


	public void btdClose_actionPerformed(){

		this.dispose();

	}


} //class: PropDialog
