/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui;

import gui.tray.*;
import map.*;
import util.*;
import controller.*;
import transport.*;
import transport.protocol.*;
import agent.*;
import agent.autonomous.*;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;



/* ===================================================================== */
/** This class is an extension of a JPanel.  It's backgound is that of
 *  scale map corresponding to real world GPS coordinates.  Agents are drawn
 *  on the JPanel, and can be dragged around the map. also wirelss ranges
 *  are drawn on each Agent if requested by the iSIM user.
 *
 */
public class SensorPanel extends JPanel implements MouseListener, ChangeListener{



	/** The controller of the simulation. */
	private Controller controller;

	/** Layout used in this panel. */
    private BorderLayout bLayMain = new BorderLayout();

    /** Layout used in this panel. */
    private GridBagLayout gbagLayInfo = new GridBagLayout();


    public JCheckBox location = new JCheckBox(" Faulty.");
    public JCheckBox bio = new JCheckBox(" Faulty.");
    public JCheckBox room = new JCheckBox(" Faulty.");
    public JCheckBox wireless = new JCheckBox(" Faulty.");

    public JCheckBox location2 = new JCheckBox(" Disable.");
    public JCheckBox bio2 = new JCheckBox(" Disable.");
    public JCheckBox room2 = new JCheckBox(" Disable.");
    public JCheckBox wireless2 = new JCheckBox(" Disable.");


	/** Holds the map background image. */
	private JLabel tab;

	private JPanel pnlMain = new JPanel();;

	private boolean open = false;

	private PropDialog parent;

	private int size;


    /* ===================================================================== */
    /** The constructor sets up mouse listeners and sets up its own variables
     *  with the given ones.
     *
     *  @param  controller   The controller of simulation.
     *  @param  ranges   True if displaying wireless ranges.
     *
     */
  	public SensorPanel(Controller c, PropDialog p){

		controller = c;
		parent = p;


		// Sign up listeners

		init();

		tab = new JLabel( new ImageIcon("files/photos/tab.gif"));
		this.add(tab, BorderLayout.CENTER);

		tab.addMouseListener(this);

		new RefreshThread(this);

  	} //Constructor



	public void init(){

 		bLayMain.setHgap(5);
        bLayMain.setVgap(5);
        pnlMain.setBorder(TTBorder.getBorder("User Sensors"));
        this.setLayout(bLayMain);
        pnlMain.setLayout(new BorderLayout());




		JLabel lblloc = new JLabel("Location sensor:");
		JLabel lblbio = new JLabel("Bio sensor:");
		JLabel lblroom = new JLabel("Room sensor:");
		JLabel lblwireless = new JLabel(" Wireless device:");

		location.addChangeListener(this);
		bio.addChangeListener(this);
		wireless.addChangeListener(this);
		room.addChangeListener(this);

		// Add stuff to panels
		JPanel stats = new JPanel();
		stats.setLayout(gbagLayInfo);


        stats.add(lblloc,    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblbio,     new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblroom,    new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblwireless,   new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));




        stats.add(location,    new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(bio,     new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(room,    new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(wireless,   new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));


        stats.add(location2,    new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(bio2,     new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(room2,    new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(wireless2,   new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));


        pnlMain.add(stats, BorderLayout.CENTER);
        this.add(pnlMain, BorderLayout.EAST);
        open = true;

	}




    /* ===================================================================== */
    /** This method checks for two kinds of mouse clicks.  If first checks for
     *  a right mouse click which is for rotating a user.(changing user's
     *  facing direction)
     *  This mehtod also checks for mouse left button double clicks.  Based on
     *  the location of click and the timing, it will lauch a PropDialog for
     *  the User that was double clicked on.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mouseClicked(MouseEvent e){

		int btn = e.getButton();
		Point p = e.getPoint();

		//possibly check location!

		if( btn == 1 ){

			if( open ){

				open = false;
				System.out.println("closing....");
				try{this.remove(pnlMain);}catch(Exception exc){}
				parent.setSize(parent.getSize().width-288, parent.getSize().height);

			}else{
				open = true;
				System.out.println("opening....");
				try{this.add(pnlMain, BorderLayout.EAST);}catch(Exception exc){}
				parent.setSize(parent.getSize().width+288, parent.getSize().height);

			}

		}


	} //method: mouseClicked


	public void mousePressed(MouseEvent e){

	} //method: mousePressed


	public void mouseReleased(MouseEvent e){
	} //method: mouseReleased

	public void mouseEntered(MouseEvent e){
	} //method: mouseReleased

	public void mouseExited(MouseEvent e){
	} //method: mouseReleased


	public void stateChanged(ChangeEvent e){

		//save the faulty states to agent.
		Agent a = controller.getCurrentAgent();

		if( !(a instanceof Fire) && a instanceof User){
			a.setBioSensor( !bio.isSelected() );
			a.setLocationSensor( !location.isSelected() );
			a.setRoomSensor( !room.isSelected() );
			a.setWirelessSensor( !wireless.isSelected() );

			if( a.isBioSensor2() && bio2.isSelected() ){
				SocketSender s1 = new SocketSender( new ExceptionMessage( ((User)a).getPort(),2323,new BioMessage(0,0,0,0,0 )));
				s1.start();
			}else if( a.isRoomSensor2() && room2.isSelected() ){
				SocketSender s1 = new SocketSender( new ExceptionMessage( ((User)a).getPort(),2323,new RoomMessage(0,0,null )));
				s1.start();
			}else if( a.isLocationSensor2() && location2.isSelected() ){
				SocketSender s1 = new SocketSender( new ExceptionMessage( ((User)a).getPort(),2323,new NMEAMessage(0,0,0,0,0,0)));
				s1.start();
			}else if( a.isWirelessSensor2() && wireless2.isSelected() ){
				SocketSender ss = new SocketSender(new ExceptionMessage( ((User)a).getPort(),2323, new MulticastMessage("",0)));
				ss.start();
			}


			a.setBioSensor2( !bio2.isSelected() );
			a.setLocationSensor2( !location2.isSelected() );
			a.setRoomSensor2( !room2.isSelected() );
			a.setWirelessSensor2( !wireless2.isSelected() );
		}


	}

	class RefreshThread extends Thread{
		JPanel p;

	  	public RefreshThread(JPanel p){
	           this.p = p;
	           this.start();
	   	} //Constructor

	  	public void run(){
	       	try{ Thread.sleep(50); }catch(Exception e){}

			open = false;
			try{p.remove(pnlMain);}catch(Exception exc){}
			parent.setSize(parent.getSize().width-288, parent.getSize().height);


	    } //method: run
	}



} //class: ChartComp

