/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package transport;

import transport.protocol.*;
import agent.*;
import controller.*;

import java.io.*;
import java.net.*;
import java.util.*;



/* ===================================================================== */
/** This class is used for Multicasting a message to all users in wireless
 *  range.  It adds in timing delays based on ratio of max range/distance
 *  to simulate bandwidth loss and delay due to wireless signal
 *  degradation.
 *
 */
public class Multicast extends Thread{

	/** The message to send in this thread. */
	private P2PMessage msg;

	/** The list of all access points in the simulation. */
	private Vector points = new Vector();

	/** The list of all users in the simulation. */
	private Vector users = new Vector();



	/* ================================================================ */
	/** This Costructor preps the Thread for the given message to send.
	 *
	 *  @param  msg   The message to send to other users in range.
	 *
	 */
	public Multicast(Vector users, Vector points, P2PMessage msg){

		this.users = users;
		this.points = points;
		this.msg = msg;

	} //Constructor


	/* ================================================================ */
	/** This method excutes the thread.  It sends the message to all
	 *  Users in range, adds a delay, and sends to Users that be reached
	 *  in a number of hops specified by the message.
	 *
	 */
	public void run(){

		System.out.println("Processing a Multicast message...");

		int hops = msg.getHops();
		int from = msg.getFrom();

		double diff = 0;
		Agent a = null;
		Agent temp = null;
		Vector tosendto = new Vector();

		// Find the agent that sent the message.
		for( int i=0; i <users.size(); i++ ){
			temp = (Agent)users.elementAt(i);
			if( from == ((User)temp).getPort() ){
				a = temp;
			}
		}

		tosendto.add(a);

		System.out.println("   sent by "+ ((User)a).getPort());
		if( !a.isWirelessSensor() ){
			SocketSender ss = new SocketSender(new ErrorMessage(((User)a).getPort(),2323, msg));
			ss.start();
		}else{


			try{

			//Now send message to all agents except sender.
			if( a != null ){

				// Send to all agents in one hop.
				for( int i=0; i <users.size(); i++ ){

					temp = (Agent)users.elementAt(i);
					diff = Controller.diff(a.getLocation(),temp.getLocation());

					//System.out.println( "    diff: "+diff);

					if( (diff < (a.getRange()+temp.getRange())) && !(tosendto.contains(temp)) && (((User)temp).getPort() != ((User)a).getPort()) ){

						// Impose a delay based on the distance and range.
						diff += 50;
						try{
							Thread.sleep( (int)(1750*(diff/(a.getRange()+temp.getRange()))) );
						}catch(Exception e){}

						System.out.println( "      sending message to: "+((User)temp).getPort());

						msg.setTo(((User)temp).getPort());
						if( !a.isWirelessSensor() ){
							SocketSender ss = new SocketSender(new ErrorMessage(((User)temp).getPort(),2323, msg));
							ss.start();
						}else if( a.isWirelessSensor2() ){
							SocketSender ss = new SocketSender(msg);
							ss.start();
						}
						tosendto.add(temp);
					}
				}
				for( int i=0; i <points.size(); i++ ){

					temp = (Agent)points.elementAt(i);
					diff = Controller.diff(a.getLocation(),temp.getLocation());

					if( diff < (a.getRange()+temp.getRange()) && !(tosendto.contains(temp)) ){
						tosendto.add(temp);
					}
				}


				// Now find those within hops.
				int h = 1;
				while( h < hops ){
					h++;

					// Impose a delay inbetween hops
					try{
						Thread.sleep(1000);
					}catch(Exception e){}

					for( int j=0; j < tosendto.size(); j++ ){

						a = (Agent)tosendto.elementAt(j);

						// Find all agents in one hop.
						for( int i=0; i <users.size(); i++ ){

							temp = (Agent)users.elementAt(i);
							diff = Controller.diff(a.getLocation(),temp.getLocation());

							if( !(tosendto.contains(temp)) && diff < (a.getRange()+temp.getRange()) ){

								// Impose a delay based on the distance and range.
								diff += 50;
								try{
									Thread.sleep( (int)(1750*(diff/(a.getRange()+temp.getRange()))) );
								}catch(Exception e){}

								System.out.println( "      sending message to: "+((User)temp).getPort());

								msg.setTo(((User)temp).getPort());
								if( !a.isWirelessSensor() ){
									SocketSender ss = new SocketSender(new ErrorMessage(((User)temp).getPort(),2323, msg));
									ss.start();
								}else if( a.isWirelessSensor2() ){
									SocketSender ss = new SocketSender(msg);
									ss.start();
								}

								tosendto.add(temp);
							}
						}
						for( int i=0; i <points.size(); i++ ){

							temp = (Agent)points.elementAt(i);
							diff = Controller.diff(a.getLocation(),temp.getLocation());

							if( !(tosendto.contains(temp)) && diff < (a.getRange()+temp.getRange()) ){
								tosendto.add(temp);

							}
						}

					}
				} //while
			}

			}catch(Exception e){System.out.println("Error in message sending method of multicast "+e);}

		}
	} //method: run

} //class: MessageSender
